/****************************************************************************

  ANALOG DEVICES
  EUROPEAN DSP APPLICATIONS

  FLASH IT! 
  
  	will write application code from data segment to the FLASH

  History:
   1.0.1.0	18-JAN-98	HS
   1.0.1.1	21-JAN-98	HS

 ****************************************************************************/

/****************************************************************************
  Global defines: change upon different codes
 ****************************************************************************/
#define f_size 1599		/* number of lines in blink.ldr */

/****************************************************************************
  Global include files
 ****************************************************************************/
#include <def21061.h>
#include "flash.h"

/****************************************************************************
  Interrupt vector table
 ****************************************************************************/
.section/pm seg_rth;
	
	nop; nop; nop; nop;			/*        reserved */
	nop; jump start; rti; rti;		/* RSTI   reset vector */
	nop; nop; nop; nop;			/*        reserved */
	rti; rti; rti; rti;			/* SOVFI  stack overflow */
	rti; rti; rti; rti;			/* TMZHI  high timer */
	rti; rti; rti; rti;			/* VIRPTI vector interrupt */
	rti; rti; rti; rti;			/* IRQ2I  irq2 vector */
	rti; rti; rti; rti;			/* IRQ1I  irq1 vector */
	rti; rti; rti; rti;			/* IRQ0I  irq0 vector */
	nop; nop; nop; nop;
	rti; rti; rti; rti;			/* SPR0I  DMA0 SP0_RX */
	rti; rti; rti; rti;			/* SPR1I  DMA1 SP1_RX */
	rti; rti; rti; rti;			/* SPT0I  DMA2 SP0_TX */
	rti; rti; rti; rti;			/* SPT1I  DMA3 SP1_TX */
	rti; rti; rti; rti;			/* LP2I   DMA4 link buffer 2 */
	rti; rti; rti; rti;			/* LP3I   DMA5 link buffer 3 */
	rti; rti; rti; rti;			/* EP0I   DMA6 ext.port buf 0 */
	rti; rti; rti; rti;			/* EP1I   DMA7 ext.port buf 1 */
	rti; rti; rti; rti;			/* EP2I   DMA8 ext.port buf 2 */
	rti; rti; rti; rti;			/* EP3I   DMA9 ext.port buf 3 */
	rti; rti; rti; rti;			/* LSRQ   link port service req */
	rti; rti; rti; rti;			/* CB7I   circ buffer 7 overflow */
	rti; rti; rti; rti;			/* CB15I  circ buffer 15 overflow */
	rti; rti; rti; rti;			/* TMZLI  low timer */
	rti; rti; rti; rti;			/* FIXI   fixed overflow */
	rti; rti; rti; rti;			/* FLTOI  floating overflow */
	rti; rti; rti; rti;			/* FLTUI  floating underflow */
	rti; rti; rti; rti;			/* FLTII  floating invalid exception */
	rti; rti; rti; rti;			/* SFT0I  software irq 0 */
	rti; rti; rti; rti;			/* SFT1I  software irq 1 */
	rti; rti; rti; rti;			/* SFT2I  software irq 2 */
	rti; rti; rti; rti;			/* SFT3I  software irq 3 */


/****************************************************************************
  File to be flashed out
 ****************************************************************************/
.section/dm seg_dmda;
.var f_data[f_size] = "blink.ldr";



/****************************************************************************
  Flash Code Section
 ****************************************************************************/
.section/pm seg_pmco;

start:	irptl = 0;		/* clear any pending interrupts */
	nop;
	bit set mode2 FLG2O | FLG1O | FLG0O | FLG3O;
	bit clr astat ASTAT_FLG3 | ASTAT_FLG2 | ASTAT_FLG1 | ASTAT_FLG0;

	r1 = 0xb000;		/* ADSP-21061: adjusting MSIZE */
	r0 = dm(SYSCON);
	r0 = r0 OR r1;
	dm(SYSCON) = r0;

	call flash_setup;		/* flash setup */

	r0 = 0x0;			/* erase sector 0 */
	dm(flash_address) = r0;
	call sect_erase;
					/* CAVEAT:
					 * 
					 * depending on code size, 
					 * multiple sectors in the 
					 * FLASH must be erased;
					 * easier but lengthly:
					 *   CHIP_ERASE()
					
 */
restart:

	i0 = f_data;		/* pointer to data */
	l0 = 0x0;


	r2 = 0x0; 			/* set address to 0 */

	
	r0 = dm(i0,1);		/* fetch new 16bit */
	r1 = fext r0 by 0:8;	/* get lower 8 bits */

/****************************************************************************
  Flash Programming Loop
 ****************************************************************************/
	lcntr = f_size, do prog_loop until lce;			
		
			dm(d_byte) = r1;
			dm(flash_address) = r2;
			call prog_byte;
			r2 = r2 + 1;

			r1 = fext r0 by 8:8;	/* get higher 8 bits */
			dm(d_byte) = r1;
			dm(flash_address) = r2;
			call prog_byte;		/* may not be placed in last */ 
						/* three cycles of a loop    */
			r2 = r2 + 1;
			r0 = dm(i0,1);		/* fetch new 16bit */
	prog_loop:	r1 = fext r0 by 0:8;	/* get lower 8 bits */

done:	r0=0x1;
	dm(SYSCON)=r0;				/* force reboot from EPROM */
